import * as i0 from '@angular/core';
import { EventEmitter, Component, Input, Directive, Output, ViewEncapsulation, Injectable, NgModule } from '@angular/core';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';
import { Subject } from 'rxjs';

var DeviceOS;
(function (DeviceOS) {
    DeviceOS[DeviceOS["Android"] = 0] = "Android";
    DeviceOS[DeviceOS["iOS"] = 1] = "iOS";
    DeviceOS[DeviceOS["Desktop"] = 2] = "Desktop";
})(DeviceOS || (DeviceOS = {}));

/**
 * This class should be extended by all SQ UI custom
 * form controls so that they get recognized by Angular forms.
 * Its only purpose is to implement the ControlValueAccessor interface.
 **/
class ControlValueAccessorEnabler {
    constructor() {
        this._modelToViewChange = new EventEmitter();
        this._onChange = () => { };
        this._onTouched = () => { };
    }
    get value() {
        return this._value;
    }
    set value(newValue) {
        if (newValue !== this._value) {
            this._value = newValue;
            this._onChange(newValue);
        }
    }
    onBlur() {
        this._onTouched();
    }
    writeValue(newValue) {
        if (newValue !== this._value) {
            this._value = newValue;
            this._modelToViewChange.emit(newValue);
        }
    }
    registerOnChange(fn) {
        this._onChange = fn;
    }
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
}

/**
 * This class contains all the base properties
 * needed for every possible SQ UI form control
 **/
class InputCoreComponent extends ControlValueAccessorEnabler {
    constructor() {
        super();
        this.defaultInputIdentifier = 'sq-form-control' + new Date().getTime().toString();
        this.name = this.defaultInputIdentifier;
        this.controlId = this.defaultInputIdentifier;
        this.controlLabel = '';
        this.controlPlaceholder = '';
        this.required = false;
        this.pattern = '';
        this.disabled = false;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: InputCoreComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.0.4", type: InputCoreComponent, selector: "ng-component", inputs: { name: "name", controlId: "controlId", controlLabel: "controlLabel", controlPlaceholder: "controlPlaceholder", required: "required", pattern: "pattern", disabled: "disabled" }, usesInheritance: true, ngImport: i0, template: '', isInline: true }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: InputCoreComponent, decorators: [{
            type: Component,
            args: [{
                    template: ''
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { name: [{
                type: Input
            }], controlId: [{
                type: Input
            }], controlLabel: [{
                type: Input
            }], controlPlaceholder: [{
                type: Input
            }], required: [{
                type: Input
            }], pattern: [{
                type: Input
            }], disabled: [{
                type: Input
            }] } });

class OutsideClickListenerDirective {
    constructor(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.clickOutside = new EventEmitter();
        this.listenForOutsideClick = false;
        this.listener = this.renderer.listen('document', 'click', (event) => {
            if (this.listenForOutsideClick) {
                const clickedInside = this.elementRef.nativeElement.contains(event.target);
                if (!clickedInside) {
                    this.clickOutside.emit();
                }
            }
        });
    }
    ngOnDestroy() {
        this.listener();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: OutsideClickListenerDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.4", type: OutsideClickListenerDirective, selector: "[sqOutsideClickListener]", inputs: { listenForOutsideClick: "listenForOutsideClick" }, outputs: { clickOutside: "clickOutside" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: OutsideClickListenerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[sqOutsideClickListener]'
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }]; }, propDecorators: { clickOutside: [{
                type: Output
            }], listenForOutsideClick: [{
                type: Input
            }] } });

class PaginatorComponent {
    constructor() {
        this.items = [];
        this.itemsPerPage = 10;
        this.currentPage = 1;
        this.maxDisplayedPages = 3;
        this.paginatedCollection = [];
        this.paginatedCollectionChange = new EventEmitter();
        this.pageChange = new EventEmitter();
        this.pages = [];
        this._paginatedCollection = [];
        this.disableNextBtns = false;
        this.disablePrevBtns = true;
        this.currentPageNumber = 1;
        this.hasSelectedCurrentPageByAuthor = false;
    }
    ngOnInit() {
        this.itemsPerPage = this.itemsPerPage || 10;
        this.maxDisplayedPages = this.maxDisplayedPages || 3;
    }
    ngOnChanges(changesObj) {
        if (changesObj.items && changesObj.items.currentValue) {
            this.generatePaginatedCollection(this.currentPageNumber);
            this.updatePageCount(this.lastPage);
            if (this.currentPage && !this.hasSelectedCurrentPageByAuthor) {
                this.selectCurrentPageProgramatically();
            }
            this.toggleControlEnabling();
        }
        if (changesObj.itemsPerPage && changesObj.itemsPerPage.currentValue &&
            changesObj.itemsPerPage.currentValue > 0) {
            this.updatePageCount(this.lastPage);
        }
        if (changesObj.currentPage && changesObj.currentPage.currentValue &&
            changesObj.currentPage.currentValue > 0) {
            this.hasSelectedCurrentPageByAuthor = false;
            this.selectCurrentPageProgramatically();
        }
        if (changesObj.lastPage && changesObj.lastPage.currentValue &&
            changesObj.lastPage.currentValue > 0) {
            this.updatePageCount(changesObj.lastPage.currentValue);
        }
        if (changesObj.maxDisplayedPages && changesObj.maxDisplayedPages.currentValue &&
            changesObj.maxDisplayedPages.currentValue > 0) {
            this.updatePageCount(this.lastPage);
        }
    }
    onPageClick(page) {
        this.selectPage(page);
        this.pageChange.emit({
            page: page.number,
            firstItemIndex: this.items.indexOf(this._paginatedCollection[0])
        });
    }
    navigateToPage(newPageDifference) {
        const selectedItemIndex = this.pages.findIndex((pageItem) => {
            return pageItem.isSelected === true;
        });
        const newPageItem = this.pages[selectedItemIndex + newPageDifference];
        this.onPageClick(newPageItem);
    }
    updatePageCount(lastPage) {
        const pageCount = lastPage || Math.ceil(this.items.length / this.itemsPerPage);
        this.pages = [];
        if (!pageCount || this.items.length === 0) {
            return;
        }
        if (pageCount === 1) {
            this.pages.push({
                number: 1,
                isSelected: false,
                isHidden: false
            });
        }
        else {
            for (let i = 1; i <= pageCount; i++) {
                this.pages.push({
                    number: i,
                    isSelected: false,
                    isHidden: true
                });
            }
        }
        const selectedItem = this.pages.find((pageItem) => {
            return pageItem.number === this.currentPageNumber;
        });
        selectedItem.isSelected = true;
        this.hidePages();
    }
    selectPage(page) {
        const previousPage = this.pages.find((pageItem) => {
            return pageItem.isSelected === true;
        });
        previousPage.isSelected = false;
        page.isSelected = true;
        this.toggleControlEnabling(page);
        this.generatePaginatedCollection(page.number);
    }
    generatePaginatedCollection(newPage) {
        let newUpperLimit = this.itemsPerPage * newPage;
        let lowerLimit = newUpperLimit - this.itemsPerPage;
        if (!this.items[lowerLimit]) {
            newUpperLimit = this.items.length - 1;
            lowerLimit = newUpperLimit - this.itemsPerPage;
        }
        this._paginatedCollection = this.items.slice(lowerLimit, newUpperLimit);
        this.currentPageNumber = newPage;
        setTimeout(() => {
            this.paginatedCollectionChange.emit(this._paginatedCollection);
        }, 100);
    }
    toggleControlEnabling(page) {
        if (!page) {
            page = this.pages.find((pageItem) => {
                return pageItem.number === this.currentPageNumber;
            });
        }
        this.disablePrevBtns = (this.pages.indexOf(page) === 0);
        this.disableNextBtns = (this.pages.indexOf(page) === this.pages.length - 1);
    }
    hidePages() {
        const selectedItemIndex = this.pages.findIndex((pageItem) => {
            return pageItem.isSelected === true;
        });
        const lastVisiblePagesFromBeginning = selectedItemIndex + this.maxDisplayedPages - 1;
        const lastVisiblePagesFromEnd = this.pages.length - 1 - this.maxDisplayedPages;
        this.pages.forEach((pageItem, index) => {
            if ((index >= selectedItemIndex && index <= lastVisiblePagesFromBeginning) ||
                index === 0 || index > lastVisiblePagesFromEnd) {
                pageItem.isHidden = false;
            }
        });
    }
    selectCurrentPageProgramatically() {
        const pageToSelect = this.pages.find((page) => {
            return page.number === this.currentPage;
        });
        if (pageToSelect) {
            this.selectPage(pageToSelect);
            this.hidePages();
            this.hasSelectedCurrentPageByAuthor = true;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: PaginatorComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.0.4", type: PaginatorComponent, selector: "sq-paginator", inputs: { items: "items", itemsPerPage: "itemsPerPage", currentPage: "currentPage", lastPage: "lastPage", maxDisplayedPages: "maxDisplayedPages", paginatedCollection: "paginatedCollection" }, outputs: { paginatedCollectionChange: "paginatedCollectionChange", pageChange: "pageChange" }, usesOnChanges: true, ngImport: i0, template: "<ol class=\"paginator display-flex wrap\" *ngIf=\"pages.length > 0\">\r\n  <li>\r\n    <button type=\"button\"\r\n            [disabled]=\"disablePrevBtns\"\r\n            data-btn-type=\"first\"\r\n            (click)=\"onPageClick(pages[0])\">\r\n      <i class=\"fas fa-angle-double-left\" aria-hidden=\"true\"></i>\r\n    </button>\r\n  </li>\r\n  <li>\r\n    <button type=\"button\"\r\n            [disabled]=\"disablePrevBtns\"\r\n            data-btn-type=\"prev\"\r\n            (click)=\"navigateToPage(-1)\">\r\n      <i class=\"fas fa-angle-left\" aria-hidden=\"true\"></i>\r\n    </button>\r\n  </li>\r\n  <li *ngFor=\"let page of pages\"\r\n      [hidden]=\"page.isHidden\"\r\n      data-btn-type=\"page-num\"\r\n      [ngClass]=\"{\r\n        'current': page.isSelected,\r\n        'truncated': page.isHidden\r\n      }\">\r\n    <button type=\"button\"\r\n            (click)=\"onPageClick(page)\">\r\n      {{page.number}}\r\n    </button>\r\n  </li>\r\n  <li>\r\n    <button type=\"button\"\r\n            [disabled]=\"disableNextBtns\"\r\n            data-btn-type=\"next\"\r\n            (click)=\"navigateToPage(1)\">\r\n      <i class=\"fas fa-angle-right\" aria-hidden=\"true\"></i>\r\n    </button>\r\n  </li>\r\n  <li>\r\n    <button type=\"button\"\r\n            [disabled]=\"disableNextBtns\"\r\n            data-btn-type=\"last\"\r\n            (click)=\"onPageClick(pages[pages.length - 1])\">\r\n      <i class=\"fas fa-angle-double-right\" aria-hidden=\"true\"></i>\r\n    </button>\r\n  </li>\r\n</ol>\r\n", styles: [".sq ol.paginator{list-style:none;margin:0}.sq ol.paginator li{position:relative;padding:5px 8px;min-width:35px;height:32px;text-align:center;margin-right:10px;transition:all .4s ease-in}.sq ol.paginator li.current{font-weight:700;background-color:#b7d8d1}.sq ol.paginator li.truncated{display:none}.sq ol.paginator li:not(.truncated)+.truncated{pointer-events:none;display:block;font-size:0}.sq ol.paginator li:not(.truncated)+.truncated:before{content:\"...\";pointer-events:none;font-size:1.1rem}.sq ol.paginator li button{position:absolute;left:0;bottom:0;width:100%;height:100%;cursor:pointer}.sq ol.paginator li button[disabled]{cursor:default}.sq ol.paginator li button[disabled] .fas{color:#aba9a9}\n"], dependencies: [{ kind: "directive", type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: PaginatorComponent, decorators: [{
            type: Component,
            args: [{ selector: 'sq-paginator', encapsulation: ViewEncapsulation.None, template: "<ol class=\"paginator display-flex wrap\" *ngIf=\"pages.length > 0\">\r\n  <li>\r\n    <button type=\"button\"\r\n            [disabled]=\"disablePrevBtns\"\r\n            data-btn-type=\"first\"\r\n            (click)=\"onPageClick(pages[0])\">\r\n      <i class=\"fas fa-angle-double-left\" aria-hidden=\"true\"></i>\r\n    </button>\r\n  </li>\r\n  <li>\r\n    <button type=\"button\"\r\n            [disabled]=\"disablePrevBtns\"\r\n            data-btn-type=\"prev\"\r\n            (click)=\"navigateToPage(-1)\">\r\n      <i class=\"fas fa-angle-left\" aria-hidden=\"true\"></i>\r\n    </button>\r\n  </li>\r\n  <li *ngFor=\"let page of pages\"\r\n      [hidden]=\"page.isHidden\"\r\n      data-btn-type=\"page-num\"\r\n      [ngClass]=\"{\r\n        'current': page.isSelected,\r\n        'truncated': page.isHidden\r\n      }\">\r\n    <button type=\"button\"\r\n            (click)=\"onPageClick(page)\">\r\n      {{page.number}}\r\n    </button>\r\n  </li>\r\n  <li>\r\n    <button type=\"button\"\r\n            [disabled]=\"disableNextBtns\"\r\n            data-btn-type=\"next\"\r\n            (click)=\"navigateToPage(1)\">\r\n      <i class=\"fas fa-angle-right\" aria-hidden=\"true\"></i>\r\n    </button>\r\n  </li>\r\n  <li>\r\n    <button type=\"button\"\r\n            [disabled]=\"disableNextBtns\"\r\n            data-btn-type=\"last\"\r\n            (click)=\"onPageClick(pages[pages.length - 1])\">\r\n      <i class=\"fas fa-angle-double-right\" aria-hidden=\"true\"></i>\r\n    </button>\r\n  </li>\r\n</ol>\r\n", styles: [".sq ol.paginator{list-style:none;margin:0}.sq ol.paginator li{position:relative;padding:5px 8px;min-width:35px;height:32px;text-align:center;margin-right:10px;transition:all .4s ease-in}.sq ol.paginator li.current{font-weight:700;background-color:#b7d8d1}.sq ol.paginator li.truncated{display:none}.sq ol.paginator li:not(.truncated)+.truncated{pointer-events:none;display:block;font-size:0}.sq ol.paginator li:not(.truncated)+.truncated:before{content:\"...\";pointer-events:none;font-size:1.1rem}.sq ol.paginator li button{position:absolute;left:0;bottom:0;width:100%;height:100%;cursor:pointer}.sq ol.paginator li button[disabled]{cursor:default}.sq ol.paginator li button[disabled] .fas{color:#aba9a9}\n"] }]
        }], ctorParameters: function () { return []; }, propDecorators: { items: [{
                type: Input
            }], itemsPerPage: [{
                type: Input
            }], currentPage: [{
                type: Input
            }], lastPage: [{
                type: Input
            }], maxDisplayedPages: [{
                type: Input
            }], paginatedCollection: [{
                type: Input
            }], paginatedCollectionChange: [{
                type: Output
            }], pageChange: [{
                type: Output
            }] } });

class CustomEventBroadcasterService {
    constructor() {
        this.broadcasterSubject = new Subject();
        this.broadcasterSubjectAsObservable = this.broadcasterSubject.asObservable();
    }
    subscribeFor(eventName, callback) {
        return this.broadcasterSubjectAsObservable
            .subscribe((customEvent) => {
            if (customEvent.broadcastEvent === eventName) {
                callback(customEvent.eventDetails);
            }
        });
    }
    broadcastEvent(eventName, eventDetails) {
        this.broadcasterSubject.next({ broadcastEvent: eventName, eventDetails: eventDetails });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: CustomEventBroadcasterService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: CustomEventBroadcasterService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: CustomEventBroadcasterService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return []; } });

class OSDetectorService {
    constructor() { }
    static getDeviceOS() {
        const userAgent = navigator.userAgent || navigator.vendor;
        let agent = DeviceOS.Desktop;
        if (/android/i.test(userAgent)) {
            agent = DeviceOS.Android;
        }
        if (/iPad|iPhone|iPod/.test(userAgent)) {
            agent = DeviceOS.iOS;
        }
        return agent;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: OSDetectorService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: OSDetectorService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: OSDetectorService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return []; } });

class ScrolledToBottomListenerDirective {
    constructor(elementRef, renderer, ngZone, cd) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.ngZone = ngZone;
        this.cd = cd;
        this.scrolledToBottom = new EventEmitter();
        this.ngZone.runOutsideAngular(() => {
            this.listener = this.renderer.listen(this.elementRef.nativeElement, 'scroll', () => {
                this.checkIfHasScrolledToBottom(this.elementRef.nativeElement);
            });
        });
    }
    checkIfHasScrolledToBottom(element) {
        const hasScrolledToBottom = element.scrollTop > 0 ?
            (Math.ceil(element.scrollHeight - element.scrollTop) <= Math.ceil(element.clientHeight + 3)) : false;
        if (hasScrolledToBottom) {
            this.scrolledToBottom.emit();
            this.cd.detectChanges();
        }
    }
    ngOnDestroy() {
        this.listener();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: ScrolledToBottomListenerDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: i0.NgZone }, { token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.4", type: ScrolledToBottomListenerDirective, selector: "[sqScrolledToBottomListener]", outputs: { scrolledToBottom: "scrolledToBottom" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: ScrolledToBottomListenerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[sqScrolledToBottomListener]'
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: i0.NgZone }, { type: i0.ChangeDetectorRef }]; }, propDecorators: { scrolledToBottom: [{
                type: Output
            }] } });

class NgSqCommonModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgSqCommonModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.0.4", ngImport: i0, type: NgSqCommonModule, declarations: [OutsideClickListenerDirective,
            PaginatorComponent,
            ScrolledToBottomListenerDirective,
            InputCoreComponent], imports: [CommonModule], exports: [OutsideClickListenerDirective,
            PaginatorComponent,
            ScrolledToBottomListenerDirective,
            InputCoreComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgSqCommonModule, providers: [
            OSDetectorService,
            CustomEventBroadcasterService
        ], imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgSqCommonModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        CommonModule
                    ],
                    providers: [
                        OSDetectorService,
                        CustomEventBroadcasterService
                    ],
                    declarations: [
                        OutsideClickListenerDirective,
                        PaginatorComponent,
                        ScrolledToBottomListenerDirective,
                        InputCoreComponent
                    ],
                    exports: [
                        OutsideClickListenerDirective,
                        PaginatorComponent,
                        ScrolledToBottomListenerDirective,
                        InputCoreComponent
                    ]
                }]
        }] });

/*
 * Public API Surface of ng-sq-common
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ControlValueAccessorEnabler, CustomEventBroadcasterService, DeviceOS, InputCoreComponent, NgSqCommonModule, OSDetectorService, OutsideClickListenerDirective, PaginatorComponent, ScrolledToBottomListenerDirective };
//# sourceMappingURL=sq-ui-ng-sq-common.mjs.map
